#include <stdio.h>
#include "edgecases.h"

int extInt1 = 101101;

char *eString = "This is an extern char string\n";

int *getLocalIntPointer(int i)
{
	return &i;
}

int *getStaticIntPointer()
{
	static int j = 8787;
	return &j;
}

int *returnPassedIntPointer(int *i)
{
	return i;
}

void printValueAtPassedIntPointer(int *i)
{
	printf("printValueAtPassedIntPointer: value at passed int pointer = %i\n", *i);
}

void ptrPrintValuesInSingleTwoIntStruct(struct twoIntStruct *tis)
{
	printf("ptrPrintValuesInSingleTwoIntStruct: i1 = %i, i2 = %i\n", tis->i1, tis->i2);
}

void printValuesInSingleTwoIntStruct(struct twoIntStruct tis)
{
	printf("printValuesInSingleTwoIntStruct: i1 = %i, i2 = %i\n", tis.i1, tis.i2);
}

void ptrPrintValuesInArrayOfTwoIntStructs(struct twoIntStruct (*tisArray)[3], int arrayLength)
{
	int i = 0;
	printf("ptrPrintValuesInArrayOfTwoIntStructs for %i twoIntStructs:\n", arrayLength);
	for (i = 0; i < arrayLength; i++)
	{
		printf("\t%i: i1 = %i, i2 = %i\n", i, tisArray[i]->i1, tisArray[i]->i2);
	}
}

void printValuesInArrayOfTwoIntStructs(struct twoIntStruct tisArray[3], int arrayLength)
{
	int i = 0;
	printf("printValuesInArrayOfTwoIntStructs for %i twoIntStructs:\n", arrayLength);
	for (i = 0; i < arrayLength; i++)
	{
		printf("\t%i: i1 = %i, i2 = %i\n", i, tisArray[i].i1, tisArray[i].i2);
	}
}

void printValuesInArrayOfPointersToTwoIntStructs(struct twoIntStruct *tisArray[3], int arrayLength)
{
	int i = 0;
	printf("printValuesInArrayOfPointersToTwoIntStructs for %i twoIntStructs:\n", arrayLength);
	for (i = 0; i < 3; i++)
	{
		//printf("\ti%i: address = %#010x\n", tisArray[i]);
		printf("\ti%i: address = %p\n", i, tisArray[i]);
	}
}

volatile int printAndReturnVolatileInt(volatile int i)
{
	printf("printAndReturnVolatileInt: i = %i\n", i);
	return i;
}

void printAGlobalString()
{
	printf("printAGlobalString: %s\n", eString);
}

void printAHardcodedString()
{
	printf("printAHardcodedString: some hardcoded text\n");
}

void callVoidReturningFunctionWithNoParams(void (*vr_func)())
{
	(*vr_func)();
}

void callPAGSViaWrapper()
{
	printf("callPAGSViaWrapper(): ");
	callVoidReturningFunctionWithNoParams(&printAGlobalString);
	printf("callPAGSViaWrapper() [second call]: ");
	callVoidReturningFunctionWithNoParams(&printAHardcodedString);
}

int addTwoInts(int a, int b)
{
	return a + b;
}

MathFunction getMathFunction(int x, int y)
{
	return &addTwoInts;
}

int doMathViaFunctionPointerRV(int u, int v)
{
	int result = 0;
	
	int (*mf)(int q, int r) = getMathFunction(u, v);
	result = (*mf)(u, v);
	
	return result;
}

void printValuesInStructWithNestedStructs(structWithNestedStructs sns)
{
	printf("printValuesInStructWithNestedStructs():\n");
	printf("\t.i1 = %i\n", sns.i1);
	printf("\t.i2 = %i\n", sns.i2);
	printf("\t.x.j1 = %i\n", sns.x.j1);
	printf("\t.x.j2 = %i\n", sns.x.j2);
	printf("\t.y.k1 = %i\n", sns.y.k1);
	printf("\t.y.k2 = %i\n", sns.y.k2);
}

void ptrPrintValuesInStructWithNestedStructs(structWithNestedStructs *sns)
{
	printf("ptrPrintValuesInStructWithNestedStructs():\n");
	printf("\t->i1 = %i\n", sns->i1);
	printf("\t->i2 = %i\n", sns->i2);
	printf("\t->x.j1 = %i\n", sns->x.j1);
	printf("\t->x.j2 = %i\n", sns->x.j2);
	printf("\t->y.k1 = %i\n", sns->y.k1);
	printf("\t->y.k2 = %i\n", sns->y.k2);
}

void printValuesInCloneStructWithNestedStructs(cloneStructWithNestedStructs clone)
{
	printf("printValuesInCloneStructWithNestedStructs():\n");
	printf("\t.i1 = %i\n", clone.i1);
	printf("\t.i2 = %i\n", clone.i2);
	printf("\t.x.j1 = %i\n", clone.x.j1);
	printf("\t.x.j2 = %i\n", clone.x.j2);
	printf("\t.y.k1 = %i\n", clone.y.k1);
	printf("\t.y.k2 = %i\n", clone.y.k2);
}

void ptrPrintValuesInCloneStructWithNestedStructs(cloneStructWithNestedStructs *clone)
{
	printf("ptrPrintValuesInCloneStructWithNestedStructs():\n");
	printf("\t->i1 = %i\n", clone->i1);
	printf("\t->i2 = %i\n", clone->i2);
	printf("\t->x.j1 = %i\n", clone->x.j1);
	printf("\t->x.j2 = %i\n", clone->x.j2);
	printf("\t->y.k1 = %i\n", clone->y.k1);
	printf("\t->y.k2 = %i\n", clone->y.k2);
}

void printValuesInCloneStructWithRenamedNestedStructs(cloneStructWithRenamedNestedStructs cloneRenamed)
{
	printf("printValuesInCloneStructWithRenamedNestedStructs():\n");
	printf("\t.m1 = %i\n", cloneRenamed.m1);
	printf("\t.m2 = %i\n", cloneRenamed.m2);
	printf("\t.a.p1 = %i\n", cloneRenamed.a.p1);
	printf("\t.a.p2 = %i\n", cloneRenamed.a.p2);
	printf("\t.b.t1 = %i\n", cloneRenamed.b.t1);
	printf("\t.b.t2 = %i\n", cloneRenamed.b.t2);
}

void ptrPrintValuesInCloneStructWithRenamedNestedStructs(cloneStructWithRenamedNestedStructs *cloneRenamed)
{
	printf("ptrPrintValuesInCloneStructWithRenamedNestedStructs():\n");
	printf("\t->m1 = %i\n", cloneRenamed->m1);
	printf("\t->m2 = %i\n", cloneRenamed->m2);
	printf("\t->a.p1 = %i\n", cloneRenamed->a.p1);
	printf("\t->a.p2 = %i\n", cloneRenamed->a.p2);
	printf("\t->b.t1 = %i\n", cloneRenamed->b.t1);
	printf("\t->b.t2 = %i\n", cloneRenamed->b.t2);
}
