#include <sys/types.h>	
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
/* begin: PsyQ only - comment out next four lines to compiled under e.g. Linux */
#include <libetc.h>		
#include <libgte.h>
#include <libgpu.h>
#include <libgs.h>
/* end: PsyQ only */
#include "edgecases.h"
#include "edgelord.h"

const static twoIntStruct constStaticSingleTwoIntStruct = { 123, 456 };
const static twoIntStruct csa3TwoIntStructs[3] = 
{
	{ .i1 = 321, .i2 = 456 },
	{ .i1 = 987, .i2 = 789 },
	{ .i1 = 466, .i2 = 644 }
};
const volatile int constVolatileInt = 9919;

const static char *csString = "This is a const static string\n";

extern int extInt1;

extern char *eString;

struct structWithNestedStructs globalStructWithNestedStructs;

struct cloneStructWithNestedStructs globalCloneStructWithNestedStructs;

struct cloneStructWithRenamedNestedStructs globalCloneStructWithRenamedNestedStructs;

struct structWithOneNestedStruct globalStructWithOneNestedStruct;

struct structWithNestedUnions globalStructWithNestedUnions;

union unionWithNestedStructs globalUnionWithNestedStructs;

void printAConstStaticString()
{
	printf("printAConstStaticString: csString = %s\n", csString);
}

void printVariadicStrings(int count, ...)
{
	va_list ap;
	int i;
	printf("printVariadicStrings:\n");
	va_start(ap, count);
	for (i = 0; i < count; i++)
	{
		printf("\t%s\n", va_arg(ap, char *));
	}
	va_end(ap);
}

int main()
{
	int i1 = 12345;
	int i2 = 31337;
	int i3;
	int i4;
	static int csInt = 77177;
	int *intPtr;
	int *intPtrClone;
	volatile int volInt1 = 31337 * 2;
	volatile int volInt2 = 0;
	volatile int volInt3 = 0;
	
	struct twoIntStruct singleTwoIntStruct;
	struct twoIntStruct *ptrToArrayOfThreeTwoIntStructs;
	struct twoIntStruct arrayOfThreeTwoIntStructs[3];
	struct twoIntStruct *threePtrsToArraysOfThreeTwoIntStructs[3];
	VoidFunction vrf;
	elStructA elsa;

	printf("i1 = %i\n", i1);
	printf("extInt1 = %i\n", extInt1);

	printf("csString = %s", csString);
	printf("csString = %s\n", csString);

	printf("eString = %s", eString);

	volInt2 = printAndReturnVolatileInt(volInt1);
	volInt3 = printAndReturnVolatileInt(constVolatileInt);
	printAndReturnVolatileInt(volInt2);
	printAndReturnVolatileInt(volInt3);
	
	// basic edgelord calls
	i3 = addValueAtPointerToValue(i1, &i2);
	printf("i3 = %i\n", i3);

	intPtr = getLocalIntPointer(i1);
	printf("intPtr = %i\n", *intPtr);
	printValueAtPassedIntPointer(intPtr);
	intPtrClone = returnPassedIntPointer(&i1);
	printf("intPtrClone = %i\n", *intPtrClone);
	printValueAtPassedIntPointer(intPtrClone);
	intPtr = getStaticIntPointer();
	printf("intPtr = %i\n", *intPtr);
	printValueAtPassedIntPointer(intPtr);

	/*singleTwoIntStruct = malloc(sizeof(struct twoIntStruct));
	if (singleTwoIntStruct == NULL)
	{
		printf("malloc() failed for singleTwoIntStruct\n");
	}*/
	singleTwoIntStruct.i1 = 999;
	singleTwoIntStruct.i2 = 888;

	printf("calling ptrPrintValuesInSingleTwoIntStruct(&singleTwoIntStruct)\n");
	ptrPrintValuesInSingleTwoIntStruct(&singleTwoIntStruct);
	
	printf("calling printValuesInSingleTwoIntStruct(singleTwoIntStruct)\n");
	printValuesInSingleTwoIntStruct(singleTwoIntStruct);
	
	ptrToArrayOfThreeTwoIntStructs = malloc(sizeof(struct twoIntStruct) * 3);
	/*if (ptrToArrayOfThreeTwoIntStructs == NULL)
	{
		printf("malloc() failed for ptrToArrayOfThreeTwoIntStructs\n");
	}*/
	//*ptrToArrayOfThreeTwoIntStructs = malloc(sizeof(struct twoIntStruct) * 3);
	//ptrToArrayOfThreeTwoIntStructs[0] = malloc(sizeof(struct twoIntStruct));
	ptrToArrayOfThreeTwoIntStructs[0].i1 = 101;
	ptrToArrayOfThreeTwoIntStructs[0].i2 = 202;
	//ptrToArrayOfThreeTwoIntStructs[1] = malloc(sizeof(struct twoIntStruct));
	ptrToArrayOfThreeTwoIntStructs[1].i1 = 303;
	ptrToArrayOfThreeTwoIntStructs[1].i2 = 404;
	//ptrToArrayOfThreeTwoIntStructs[2] = malloc(sizeof(struct twoIntStruct));
	ptrToArrayOfThreeTwoIntStructs[2].i1 = 505;
	ptrToArrayOfThreeTwoIntStructs[2].i2 = 606;

	printf("calling ptrPrintValuesInArrayOfTwoIntStructs(ptrToArrayOfThreeTwoIntStructs, %i)\n", 3);
	ptrPrintValuesInArrayOfTwoIntStructs(ptrToArrayOfThreeTwoIntStructs, 3);
	printf("calling printValuesInArrayOfTwoIntStructs(ptrToArrayOfThreeTwoIntStructs, %i)\n", 3);
	printValuesInArrayOfTwoIntStructs(ptrToArrayOfThreeTwoIntStructs, 3);

	printf("calling ptrPrintValuesInSingleTwoIntStruct(&constStaticSingleTwoIntStruct)\n");
	ptrPrintValuesInSingleTwoIntStruct(&constStaticSingleTwoIntStruct);
	printf("calling printValuesInSingleTwoIntStruct(constStaticSingleTwoIntStruct)\n");
	printValuesInSingleTwoIntStruct(constStaticSingleTwoIntStruct);
	printf("calling ptrPrintValuesInArrayOfTwoIntStructs(&csa3TwoIntStructs, 3)\n");
	ptrPrintValuesInArrayOfTwoIntStructs(&csa3TwoIntStructs, 3);
	printf("calling printValuesInArrayOfTwoIntStructs(csa3TwoIntStructs, 3)\n");
	printValuesInArrayOfTwoIntStructs(csa3TwoIntStructs, 3);
	
	printf("calling printValuesInArrayOfPointersToTwoIntStructs(csa3TwoIntStructs, 3)\n");
	printValuesInArrayOfPointersToTwoIntStructs(csa3TwoIntStructs, 3);

	threePtrsToArraysOfThreeTwoIntStructs[0] = malloc(sizeof(struct twoIntStruct) * 3);
	threePtrsToArraysOfThreeTwoIntStructs[1] = malloc(sizeof(struct twoIntStruct) * 3);
	threePtrsToArraysOfThreeTwoIntStructs[2] = malloc(sizeof(struct twoIntStruct) * 3);
	
	threePtrsToArraysOfThreeTwoIntStructs[0][0].i1 = 111;
	threePtrsToArraysOfThreeTwoIntStructs[0][1].i1 = 222;
	threePtrsToArraysOfThreeTwoIntStructs[0][2].i1 = 333;

	threePtrsToArraysOfThreeTwoIntStructs[1][0].i1 = 444;
	threePtrsToArraysOfThreeTwoIntStructs[1][1].i1 = 555;
	threePtrsToArraysOfThreeTwoIntStructs[1][2].i1 = 666;

	threePtrsToArraysOfThreeTwoIntStructs[2][0].i1 = 777;
	threePtrsToArraysOfThreeTwoIntStructs[2][1].i1 = 888;
	threePtrsToArraysOfThreeTwoIntStructs[2][2].i1 = 999;
	
	printf("calling printValuesInArrayOfPointersToTwoIntStructs(threePtrsToArraysOfThreeTwoIntStructs, 3)\n");
	printValuesInArrayOfPointersToTwoIntStructs(threePtrsToArraysOfThreeTwoIntStructs, 3);

	printf("calling printValuesInArrayOfPointersToTwoIntStructs(*threePtrsToArraysOfThreeTwoIntStructs, 3)\n");
	printValuesInArrayOfPointersToTwoIntStructs(*threePtrsToArraysOfThreeTwoIntStructs, 3);

	printf("calling printValuesInArrayOfPointersToTwoIntStructs(&threePtrsToArraysOfThreeTwoIntStructs, 3)\n");
	printValuesInArrayOfPointersToTwoIntStructs(&threePtrsToArraysOfThreeTwoIntStructs, 3);

	printf("calling printAConstStaticString() via callVoidReturningFunctionWithNoParams: ");
	vrf = &printAConstStaticString;
	callVoidReturningFunctionWithNoParams(vrf);
	
	printf("callPAGSViaWrapper(): ");
	callPAGSViaWrapper();

	i4 = doMathViaFunctionPointerRV(3, 11);
	printf("i4 = %i\n", i4);
	
	elsa.char1 = 37;
	elsa.ulonglong1 = 123471204571254;
	elsa.ptr = &i1;
	elsa.doublePointer = &elsa.ptr;
	elsa.triplePointer = &elsa.doublePointer;
	elsa.quadPointer = &elsa.triplePointer;
	elsa.volInt = volInt3;

	printElStructA(elsa);
	
	printVariadicStrings(3, "variadic argument 1", "variadic argument 2", "variadic argument 3");
	
	/*printf("calling printValuesInSingleConstStaticTwoIntStruct(constStaticSingleTwoIntStruct)\n");
	printValuesInSingleConstStaticTwoIntStruct(&constStaticSingleTwoIntStruct);
	printf("calling printValuesInConstStaticArrayOfTwoIntStructs(csa3TwoIntStructs, 4)\n");
	printValuesInConstStaticArrayOfTwoIntStructs(csa3TwoIntStructs, 4);*/
	
	//free(singleTwoIntStruct);
	//free(ptrToArrayOfThreeTwoIntStructs);
	
	globalStructWithNestedStructs.i1 = 797;
	globalStructWithNestedStructs.i2 = 111;
	globalStructWithNestedStructs.x.j1 = 81818;
	globalStructWithNestedStructs.x.j2 = 12345;
	globalStructWithNestedStructs.y.k1 = 63636;
	globalStructWithNestedStructs.y.k2 = 717;

	globalStructWithOneNestedStruct.i1 = 111;
	globalStructWithOneNestedStruct.i2 = 222;
	globalStructWithOneNestedStruct.x.j1 = 333;
	globalStructWithOneNestedStruct.x.j2 = 444;
	globalStructWithOneNestedStruct.x.j3 = 555;
	globalStructWithOneNestedStruct.x.j4 = 666;

	globalStructWithNestedUnions.i1 = 9999;
	globalStructWithNestedUnions.i2 = 8888;
	globalStructWithNestedUnions.x.j1 = 7777;
	globalStructWithNestedUnions.y.j2 = 6666;
	globalStructWithNestedUnions.z.k1 = 5555;
	globalStructWithNestedUnions.b.a.m1a = 4444;
	globalStructWithNestedUnions.b.a.m2b = 3333;

	globalUnionWithNestedStructs.x.i1 = 1;
	globalUnionWithNestedStructs.x.i2 = 2;
	globalUnionWithNestedStructs.x.i3 = 3;
	globalUnionWithNestedStructs.y.j1 = 4;
	globalUnionWithNestedStructs.y.j2 = 5;
	globalUnionWithNestedStructs.y.j3 = 6;
	
	printf("printValuesInStructWithNestedStructs(globalStructWithNestedStructs): \n");
	printValuesInStructWithNestedStructs(globalStructWithNestedStructs);

	printf("ptrPrintValuesInStructWithNestedStructs(&globalStructWithNestedStructs): \n");
	ptrPrintValuesInStructWithNestedStructs(&globalStructWithNestedStructs);
	
	globalCloneStructWithNestedStructs.i1 = 234;
	globalCloneStructWithNestedStructs.i2 = 567;
	globalCloneStructWithNestedStructs.x.j1 = 890;
	globalCloneStructWithNestedStructs.x.j2 = 901;
	globalCloneStructWithNestedStructs.y.k1 = 135;
	globalCloneStructWithNestedStructs.y.k2 = 357;
	
	globalCloneStructWithRenamedNestedStructs.m1 = 975;
	globalCloneStructWithRenamedNestedStructs.m2 = 753;
	globalCloneStructWithRenamedNestedStructs.a.p1 = 531;
	globalCloneStructWithRenamedNestedStructs.a.p2 = 864;
	globalCloneStructWithRenamedNestedStructs.b.t1 = 642;
	globalCloneStructWithRenamedNestedStructs.b.t2 = 420;
	
	printf("printValuesInCloneStructWithNestedStructs(globalCloneStructWithNestedStructs): \n");
	printValuesInCloneStructWithNestedStructs(globalCloneStructWithNestedStructs);

	printf("ptrPrintValuesInCloneStructWithNestedStructs(&globalCloneStructWithNestedStructs): \n");
	ptrPrintValuesInCloneStructWithNestedStructs(&globalCloneStructWithNestedStructs);
	
	printf("printValuesInCloneStructWithRenamedNestedStructs(globalCloneStructWithRenamedNestedStructs): \n");
	printValuesInCloneStructWithRenamedNestedStructs(globalCloneStructWithRenamedNestedStructs);

	printf("ptrPrintValuesInCloneStructWithRenamedNestedStructs(&globalCloneStructWithRenamedNestedStructs): \n");
	ptrPrintValuesInCloneStructWithRenamedNestedStructs(&globalCloneStructWithRenamedNestedStructs);
	
	printf("What happens if identically-defined (but differently-named) structs are passed to those functions?\n");
	
	//printf("printValuesInStructWithNestedStructs((structWithNestedStructs)globalCloneStructWithNestedStructs): \n");
	//printValuesInStructWithNestedStructs((structWithNestedStructs)globalCloneStructWithNestedStructs);

	printf("ptrPrintValuesInStructWithNestedStructs(&globalCloneStructWithNestedStructs): \n");
	ptrPrintValuesInStructWithNestedStructs(&globalCloneStructWithNestedStructs);
	
	//printf("printValuesInStructWithNestedStructs((structWithNestedStructs)globalCloneStructWithRenamedNestedStructs): \n");
	//printValuesInStructWithNestedStructs((structWithNestedStructs)globalCloneStructWithRenamedNestedStructs);

	printf("ptrPrintValuesInStructWithNestedStructs(&globalCloneStructWithRenamedNestedStructs): \n");
	ptrPrintValuesInStructWithNestedStructs(&globalCloneStructWithRenamedNestedStructs);
	
	

	
	return 0;
}